/*------------------------------------------------
 * Proyecto: CronArt
 * Autor:    Pedro Reina <pedro@pedroreina.net>
 * Archivo:  CronArtActivity.java
 * Fecha:    S.16.4.2011
 * Licencia: CC0 1.0 Universal
 *----------------------------------------------*/

// El paquete de este programa
package net.pedroreina.cronart;

// Manejar la actividad
import android.app.Activity;
import android.os.Bundle;

// Manejar el menú
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;

// Dibujar
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.view.View;

// Calcular lo que ocupa un texto
import android.graphics.Rect;

// Elegir el tipo de letra
import android.graphics.Typeface;

// Usar diálogos de alerta
import android.app.Dialog;
import android.app.AlertDialog;
import android.app.AlertDialog.Builder;

// Usar números aleatorios
import java.util.Random;

// Convertir entre enteros y cadenas
import java.lang.Integer;

// Usar listas
import java.util.ArrayList;

// Usar el reloj de precisión
import android.os.SystemClock;

// Manejar textos con etiquetas HTML
import android.text.Html;
import android.text.Spanned;

// Lanzar la actividad de las preferencias
import android.content.Intent;

// Leer las preferencias
import android.preference.PreferenceManager;
import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;

// max()
import java.lang.Math;

//-----------------------------------------------
// La actividad principal
//-----------------------------------------------
public class CronArtActivity extends Activity
  {
  AlertDialog About, History, License, Credits;
  Intent Config;

  // Called when the activity is first created
  @Override
  public void onCreate (Bundle savedInstanceState)
    {
    AlertDialog.Builder Constructor;

    super.onCreate (savedInstanceState);
    setContentView (new Dibujo(this));

    // Creamos el diálogo About
    Constructor = new AlertDialog.Builder(this);
    String Mensaje = "<b>" + getString (R.string.app_name) + "</b>";
      Mensaje += "<BR>" + getString (R.string.version);
      Mensaje += " " + getString (R.string.num_version);
      Mensaje += ", " + getString (R.string.fecha);
      Mensaje += "<BR>Pedro Reina";
      Mensaje += "<BR>http://pedroreina.net";
    Constructor.setTitle (getString (R.string.about));
    Constructor.setMessage (Html.fromHtml (Mensaje));
    Constructor.setPositiveButton (getString(R.string.OK), null);
    About = Constructor.create();

    // Creamos el diálogo License
    Constructor = new AlertDialog.Builder (this);
    Constructor.setTitle (getString (R.string.license));
    Constructor.setMessage (getText (R.string.text_license));
    Constructor.setPositiveButton (getString (R.string.OK), null);
    License = Constructor.create();

    // Creamos el diálogo Credits
    Constructor = new AlertDialog.Builder(this);
    Constructor.setTitle (getString (R.string.credits));
    Constructor.setMessage (getText (R.string.text_credits));
    Constructor.setPositiveButton (getString (R.string.OK), null);
    Credits = Constructor.create();

    // Creamos el diálogo History
    Constructor = new AlertDialog.Builder(this);
    Constructor.setTitle (getString (R.string.history));
    Constructor.setMessage (getText (R.string.text_history));
    Constructor.setPositiveButton (getString (R.string.OK), null);
    History = Constructor.create();

    // Preparamos la actividad Preferences
    Config = new Intent (getBaseContext(), CronArtPreferences.class);
    }

  // Creación del menú de opciones
  @Override
  public boolean onCreateOptionsMenu (Menu menu)
    {
    MenuInflater inflater = getMenuInflater();
    inflater.inflate (R.menu.options, menu);
    return true;
    }

  // Manejo de las contestaciones del menú de opciones
  @Override
  public boolean onOptionsItemSelected (MenuItem item)
    {
    // Actuamos segun elija el usuario
    switch ( item.getItemId() )
      {
      case R.id.config:
        startActivity (Config);
        return true;

      case R.id.history:
        History.show();
        return true;

      case R.id.license:
        License.show();
        return true;

      case R.id.credits:
        Credits.show();
        return true;

      case R.id.about:
        About.show();
        return true;

      case R.id.quit:
        // Cerramos el programa
        this.finish();
        System.runFinalizersOnExit (true);
        System.exit (0);
        return true;

      default:
        return super.onOptionsItemSelected (item);
      }
    }

  //-----------------------------------------------
  // La clase que sirve para dibujar en ella
  //-----------------------------------------------
  public class Dibujo extends View
    implements OnSharedPreferenceChangeListener
    {
    Context Contexto;

    // Conjunto de preferencias
    SharedPreferences Pref;

    // Generador de números aleatorios
    Random Ruleta = new Random();

    // Manejo de pantalla
    int Altura, Anchura, TamanoCelda;
    int MinFil, MaxFil, MinCol, MaxCol;
    int CentroX, CentroY;

    // Total de contadores que vamos a usar
    int TotalContador;

    // Lista con los contadores
    ArrayList<Integer> Contador;

    // Lista con los lapsos para cada contador
    ArrayList<Integer> Lapso;

    // Lista con el tiempo transcurrido sin actualizar un contador
    ArrayList<Integer> Transcurrido;

    // Valores del reloj
    long Antes, Ahora;

    // Modo de pintar en pantalla
    Paint Pintura = new Paint (Paint.ANTI_ALIAS_FLAG);

    // Qué hacer cuando se instancia la clase Dibujo
    public Dibujo (Context context)
      {
      super (context);
      Contexto = context;

      // Preparamos el lector de preferencias
      Pref = PreferenceManager.getDefaultSharedPreferences (Contexto);
      Pref.registerOnSharedPreferenceChangeListener (this);

      // Preparamos el modo de pintar en pantalla
      Pintura.setTextAlign (Paint.Align.CENTER);
      Pintura.setTypeface (Typeface.MONOSPACE);
      Pintura.setColor (0xFFFF0000);
      }

    // Preparamos los datos iniciales cuando sabemos las dimensiones
    // de la zona en la que se va a dibujar
    @Override
    protected void onSizeChanged (int Anc, int Alt, int OldAnc, int OldAlt)
      {
      // Guardamos las dimensiones de la zona
      Altura = Alt;
      Anchura = Anc;

      // Preparamos la fuente para escribir
      PreparaFuente();

      // Preparamos todos los valores iniciales
      PreparaValoresIniciales();

      // Y dejamos que el sistema también haga lo suyo
      super.onSizeChanged (Anc, Alt, OldAnc, OldAlt);
      }

    void PreparaFuente()
      {
      // Para leer la preferencia Tamaño
      final String TamanoTextoDefecto = "42";
      String TamanoCadena;
      int TamanoTexto;

      // Leemos el Tamaño de las preferencias
      TamanoCadena = Pref.getString ("tamano", TamanoTextoDefecto);
      TamanoTexto = new Integer (TamanoCadena);

      // Asignamos el tamaño
      Pintura.setTextSize (TamanoTexto);
      }

    void PreparaValoresIniciales()
      {
      Rect Limites = new Rect();

      // Calculamos el tamaño de la celda: debe ser cuadrada
      // y envolver dos dígitos
      Pintura.getTextBounds ("00", 0, 2, Limites);
      TamanoCelda =  Math.max (Limites.width(), Limites.height());

      // Calculamos el punto central de la pantalla
      CentroX = Anchura/2;
      CentroY = Altura/2 + TamanoCelda/2;

      // Calculamos las filas y columnas que caben en la zona
      // de modo que la celda (0,0) quede en el centro
      MaxFil = (((Altura - TamanoCelda) / 2) / TamanoCelda) / 2;
      MinFil = -MaxFil;
      MaxCol = (((Anchura - TamanoCelda) / 2) / TamanoCelda) / 2;
      MinCol = -MaxCol;

      // Calculamos cuántos contadores necesitamos
      TotalContador = (MaxFil-MinFil+1) * (MaxCol-MinCol+1);

      // Iniciamos todas las listas
      Contador = new ArrayList<Integer>();
      Lapso = new ArrayList<Integer>();
      Transcurrido = new ArrayList<Integer>();

      for ( int i=0 ; i<TotalContador ; i++ )
        {
        int Tiempo;

        // Iniciamos los contadores con números aleatorios entre 00 y 99
        Contador.add (Ruleta.nextInt (100));

        // Iniciamos los lapsos con números aleatorios entre 100 y 2000
        Tiempo = 100 + 100 * Ruleta.nextInt (19);
        Lapso.add (Tiempo);

        // Rellenamos al azar el tiempo transcurrido
        Transcurrido.add (Ruleta.nextInt (Tiempo));
        }

      // Preparamos los tiempos
      Antes = 0;
      Ahora = SystemClock.uptimeMillis();
      }

    @Override
    public void onSharedPreferenceChanged (SharedPreferences Pref, String key)
      {
      PreparaFuente();
      PreparaValoresIniciales();
      }

    // Bucle principal del programa
    @Override
    protected void onDraw (Canvas canvas)
      {
      int i, Fil, Col, Num;
      String NumTex;
      long TiempoEntreLlamadas;

      // Guardamos el tiempo anterior
      Antes = Ahora;

      // Tomamos nota del tiempo actual
      Ahora = SystemClock.uptimeMillis();

      // Vemos cuánto tiempo ha pasado
      TiempoEntreLlamadas = Ahora - Antes;

      // Recorremos todas las celdas con números
      for ( i=0 , Fil=MinFil ; Fil<=MaxFil ; Fil++ )
        {
        for ( Col=MinCol ; Col<=MaxCol ; Col++ , i++ )
          {
          // Preparamos el valor del contador
          Num = Contador.get(i);
          NumTex = Integer.toString (Num);
          if ( Num < 10 )
            { NumTex = "0" + NumTex; }

          // Escribimos el contador
          canvas.drawText (NumTex, CentroX + 2*TamanoCelda*Col,
            CentroY + 2*TamanoCelda*Fil, Pintura);

          // Aumentamos el tiempo transcurrido
          Transcurrido.set (i, (int)(Transcurrido.get(i)+TiempoEntreLlamadas));

          // Comprobamos si se ha alcanzado el Lapso
          if ( Transcurrido.get(i) >= Lapso.get(i) )
            {
            Transcurrido.set (i,0);
            // Actualizamos el valor
            Num++;
            if ( Num == 100 )  { Num = 0; }
            Contador.set (i, Num);
            }
          }
        }
      invalidate();
      }
    }
  }
